<?php

namespace Drupal\xray_audit\Plugin\xray_audit\tasks\ContentMetric;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\xray_audit\Form\SettingsForm;
use Drupal\xray_audit\Services\EntityUseInterface;
use Drupal\xray_audit\Services\PluginRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "queries_data_nodes",
 *   label = @Translation("Node reports"),
 *   description = @Translation("Metrics about node entities."),
 *   group = "content_metric",
 *   sort = 1,
 *   local_task = 1,
 *   operations = {
 *      "node_count_type" = {
 *          "label" = "Nodes grouped by type",
 *          "description" = "Number of Nodes grouped by type."
 *       },
 *       "node_count_type_langcode" = {
 *          "label" = "Nodes grouped by type and language",
 *          "description" = "Number of Nodes grouped by type and language."
 *       },
 *      "node_revisions_count" = {
 *          "label" = "Nodes grouped by revisions",
 *          "description" = "Nodes with the highest number of revisions."
 *       }
 *    },
 *   dependencies = {"node"}
 * )
 */
final class XrayAuditQueryTaskNodePlugin extends XrayAuditQueryTaskPluginBase {

  const XRA_NODE_TEMPORARY_TABLE_NAME = 'xra_nodes_hierarchy_tmp';

  /**
   * Service "xray_audit.entity_use_node".
   *
   * @var \Drupal\xray_audit\Services\EntityUseInterface
   */
  protected $serviceEntityUsenode;

  /**
   * Service "request_stack".
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Service "state".
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Service "config.factory".
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Service "module_handler".
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Configuration settings for Xray Audit.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $xrayAuditConfig;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Database connection.
   * @param \Drupal\xray_audit\Services\EntityUseInterface $entity_use_node
   *   Service "xray_audit.entity_use_node".
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Service "request_stack".
   * @param \Drupal\xray_audit\Services\PluginRepositoryInterface $xray_audit_plugin_repository
   *   Service "xray_audit.plugin_repository".
   * @param \Drupal\Core\State\StateInterface $state
   *   Service "state".
   *
   * @SuppressWarnings(PHPMD.ExcessiveParameterList)
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    Connection $database,
    LanguageManagerInterface $language_manager,
    EntityUseInterface $entity_use_node,
    RequestStack $request_stack,
    PluginRepositoryInterface $xray_audit_plugin_repository,
    StateInterface $state,
    ConfigFactoryInterface $config_factory,
    ModuleHandlerInterface $module_handler,
  ) {

    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $database, $language_manager, $xray_audit_plugin_repository, $module_handler);
    $this->serviceEntityUsenode = $entity_use_node;
    $this->requestStack = $request_stack;
    $this->state = $state;
    $this->configFactory = $config_factory;
    $this->moduleHandler = $module_handler;
    $this->xrayAuditConfig = $this->configFactory->get(SettingsForm::SETTINGS);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('database'),
      $container->get('language_manager'),
      $container->get('xray_audit.entity_use_node'),
      $container->get('request_stack'),
      $container->get('xray_audit.plugin_repository'),
      $container->get('state'),
      $container->get('config.factory'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = '') {
    $cid = $this->getPluginId() . ':' . $operation;

    $data = $this->pluginRepository->getCachedData($cid);
    if (!empty($data) && is_array($data)) {
      return $data;
    }

    switch ($operation) {
      case 'node_count_type':
        $data = $this->nodeByTypes();
        break;

      case 'node_count_type_langcode':
        $data = $this->nodeTypesPerLanguage();
        break;

      case 'node_revisions_count':
        $data = $this->nodesWithExcessiveRevisions();
        break;
    }

    $this->pluginRepository->setCacheTagsInv($cid, $data, ['node_list']);
    return $data;
  }

  /**
   * Get data for operation "node_by_types".
   *
   * @return array
   *   Render array.
   */
  protected function nodeByTypes() {

    $alias_count = 'count';
    $headerTable = [
      $this->t('ID'),
      $this->t('Label'),
      $this->t('Total'),
      $this->t('Published'),
      $this->t('Unpublished'),
    ];
    $resultTable = [];

    // Get label of content types.
    $label_content_types = [];
    $types = $this->entityTypeManager->getStorage("node_type")->loadMultiple();
    foreach ($types as $key => $type) {
      $label_content_types[$key] = $type->label();
    }

    $query = $this->entityTypeManager
      ->getStorage('node')
      ->getAggregateQuery()
      ->accessCheck(FALSE)
      ->aggregate('nid', 'count', NULL, $alias_count)
      ->groupBy('type')
      ->groupBy('status');

    $query_results = $query->execute();

    // The results are grouped in arrays and each array has
    // this data: type, status, count.
    // Then, there is two items for each content type, one
    // for published and one for unpublished.
    $total = 0;
    $total_published = 0;
    $total_unpublished = 0;
    foreach ($query_results as $result) {
      $bundle = $result['type'] ?? '';
      $status = $result['status'] ?? 0;
      $status = (int) $status;
      $count = $result[$alias_count] ?? 0;

      if (!isset($resultTable[$bundle])) {
        $resultTable[$bundle] = [
          'id' => $bundle,
          'label' => $label_content_types[$bundle] ?? $bundle,
          'total' => 0,
          'published' => 0,
          'unpublished' => 0,
        ];
      }

      $resultTable[$bundle]['total'] += $count;

      if ($status === 1) {
        $resultTable[$bundle]['published'] = $count;
      }
      elseif ($status === 0) {
        $resultTable[$bundle]['unpublished'] = $count;
      }

      $total += $count;
      $total_published += $resultTable[$bundle]['published'];
      $total_unpublished += $resultTable[$bundle]['unpublished'];
    }

    $resultTable['total']['id'] = $this->t('TOTAL');
    $resultTable['total']['label'] = '---';
    $resultTable['total']['total'] = $total;
    $resultTable['total']['published'] = $total_published;
    $resultTable['total']['unpublished'] = $total_unpublished;

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
    ];

  }

  /**
   * Get data for operation "node_by_types_language".
   *
   * @return array
   *   Render array.
   */
  protected function nodeTypesPerLanguage() {

    $alias_count = 'count';
    $header_table = [
      $this->t('ID'),
      $this->t('Label'),
      $this->t('Language'),
      $this->t('Total'),
      $this->t('Published'),
      $this->t('Unpublished'),
    ];

    // Get label of content types.
    $label_content_types = [];
    $types = $this->entityTypeManager->getStorage("node_type")->loadMultiple();
    foreach ($types as $key => $type) {
      $label_content_types[$key] = $type->label();
    }

    // The results are grouped in arrays and each array has
    // this data: type, status, count, langcode.
    // Then, there are many items for each content type,
    // for published and one for unpublished and for language.
    $query = $this->entityTypeManager
      ->getStorage("node")
      ->getAggregateQuery()
      ->accessCheck(FALSE)
      ->aggregate('nid', 'COUNT', NULL, $alias_count)
      ->groupBy('langcode')
      ->groupBy('type')
      ->groupBy('status');

    $query_results = $query->execute();
    $result_table = [];
    foreach ($query_results as $result) {

      $bundle = $result['type'] ?? '';
      $status = $result['status'] ?? 0;
      $status = (int) $status;
      $count = $result[$alias_count] ?? 0;
      $langcode = $result['langcode'] ?? '';
      $key = $bundle . '-' . $langcode;

      if (!isset($result_table[$key])) {
        $result_table[$key] = [
          'id' => $bundle,
          'label' => $label_content_types[$bundle],
          'language' => $langcode,
          'total' => 0,
          'published' => 0,
          'unpublished' => 0,
        ];
      }

      $result_table[$key]['total'] += $count;

      if ($status === 1) {
        $result_table[$key]['published'] = $count;
      }
      elseif ($status === 0) {
        $result_table[$key]['unpublished'] = $count;
      }
    }

    return [
      'header_table' => $header_table,
      'results_table' => $result_table,
    ];

  }

  /**
   * Get nodes with too many revisions.
   *
   * @return array
   *   Render array.
   */
  protected function nodesWithExcessiveRevisions() {
    $alias_count = 'count';

    $headerTable = [
      $this->t('ID'),
      $this->t('Label'),
      $this->t('Revisions'),
    ];
    $resultTable = [];

    $query = $this->database->select('node_revision', 'nr');
    $query->addField('nr', 'nid');
    $query->addExpression('COUNT(nr.vid)', $alias_count);
    $query->join('node_field_data', 'nfd', 'nr.nid = nfd.nid');
    $query->addExpression('MAX(nfd.title)', 'label');
    $query->groupBy('nr.nid');
    $query->orderBy($alias_count, 'DESC');
    $query->range(0, 20);

    $result = $query->execute()->fetchAll();

    $max_revision_count = $this->xrayAuditConfig->get('revisions_thresholds.node');

    foreach ($result as $row) {
      $nid = $row->nid;
      $label = $row->label;
      $revisions = $row->{$alias_count};

      $resultTable[$nid] = [
        'id' => $nid,
        'label' => $label,
        'revisions' => [
          'data' => $revisions,
        ],
      ];

      if (!empty($max_revision_count) && $revisions > $max_revision_count) {
        $resultTable[$nid]['revisions']['class'] = 'xray-audit-color-error';
      }
    }
    if (empty($resultTable)) {
      $resultTable[] = [
        'id' => $this->t('No nodes found'),
        'label' => '-',
        'revisions' => 0,
      ];
    }

    return [
      'header_table' => $headerTable,
      'results_table' => $resultTable,
      'extended_data_render' => [
        '#attached' => [
          'library' => [
            'xray_audit/color',
          ],
        ],
      ],
    ];
  }

}
