<?php

namespace Drupal\xray_audit\Plugin\xray_audit\tasks\Webform;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Drupal\xray_audit\Services\PluginRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation for Webform report.
 *
 * @XrayAuditTaskPlugin (
 *   id = "webform_report",
 *   label = @Translation("Webform information"),
 *   description = @Translation("Report about webforms"),
 *   group = "forms",
 *   sort = 1,
 *   local_task = 1,
 *   operations = {
 *      "webforms" = {
 *           "label" = "Webforms",
 *           "description" = "Report about webforms.",
 *           "dependencies" = {"webform"},
 *        },
 *    },
 * )
 */
class XrayAuditWebformGeneralTaskPlugin extends XrayAuditTaskPluginBase {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Service "xray_audit.plugin_repository".
   *
   * @var \Drupal\xray_audit\Services\PluginRepositoryInterface
   */
  protected $pluginRepository;

  /**
   * Cache tags for webform.
   *
   * @var array
   */
  protected $webformCachegTags = [];

  /**
   * Constructs the plugin.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    PluginRepositoryInterface $pluginRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->pluginRepository = $pluginRepository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('xray_audit.plugin_repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = '') {
    $cid = $this->getPluginId() . ':' . $operation;

    $data = $this->pluginRepository->getCachedData($cid);
    if (!empty($data) && is_array($data)) {
      return $data;
    }

    switch ($operation) {
      case 'webforms':
        $data = $this->webforms();
        break;

      default:
        return ['webform_data' => [], 'num_webforms' => 0];
    }

    $this->pluginRepository->setCacheTagsInv($cid, $data, $this->webformCachegTags);
    return $data;
  }

  /**
   * Generate report for Webforms.
   *
   * @return array
   *   Render array.
   */
  protected function webforms() {
    // Use entityTypeManager to optimize the query.
    $webformStorage = $this->entityTypeManager->getStorage('webform');

    /** @var \Drupal\webform\Entity\Webform[] $webforms */
    $webforms = $webformStorage->loadMultiple();

    if (empty($webforms)) {
      return [
        'num_webforms' => 0,
        'webform_data' => [],
      ];
    }

    $webform_data = [];
    foreach ($webforms as $webform) {
      // Get the cache tags of the webform.
      $this->webformCachegTags = Cache::mergeTags($this->webformCachegTags, $webform->getCacheTags());

      // Obtain the handlers correctly from each webform.
      $handlers = $webform->getHandlers();
      $handler_ids = array_keys($handlers->getInstanceIds());

      $url = $webform->toUrl('edit-form');
      $url->setOption('attributes', ['target' => '_blank']);
      if ($url->access()) {
        $edit_link = Link::fromTextAndUrl($this->t('Edit'), $url)->toString();
      } else
      {
        $edit_link = $this->t('No access to edit');
      }

      $webform_data[] = [
        'title' => $webform->label(),
        'id' => $webform->id(),
        'status' => $webform->isOpen() ? $this->t('Open') : $this->t('Closed'),
        'num_fields' => count($webform->getElementsDecoded()),
        'handlers' => implode(', ', $handler_ids),
        'edit_link' => $edit_link,
      ];
    }

    return [
      'num_webforms' => count($webform_data),
      'webform_data' => $webform_data,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildDataRenderArray(array $data, string $operation = '') {
    if ($operation === 'webforms' && !empty($data['webform_data'])) {
      $header = [
        $this->t('Title'),
        $this->t('ID'),
        $this->t('Status'),
        $this->t('Number of Fields'),
        $this->t('Handlers'),
        $this->t('Webform'),
      ];

      $rows = [];
      foreach ($data['webform_data'] as $webform) {
        $rows[] = [
          'title' => $webform['title'],
          'id' => $webform['id'],
          'status' => $webform['status'],
          'num_fields' => $webform['num_fields'],
          'handlers' => $webform['handlers'],
          'edit_link' => $webform['edit_link'],
        ];
      }

      return [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => $this->t('No webforms found.'),
      ];
    }

    return ['#markup' => $this->t('No data available for this operation.')];
  }

}
